//
//	Encoder for G.729A CS-ACELP Speech Codec
// 
//		adaptive_codebook_search.cpp : Module for 
//			- [A.3.7] Fixed Codebook
//

#include "StdAfx.h"
#include "dec_adaptive_codebook_search.h"

// Hamming window used for the interpolation
float b30[31] = {
(float) 0.898517,
(float) 0.769271,
(float) 0.448635,
(float) 0.095915,
(float)-0.134333,
(float)-0.178528,
(float)-0.084919,
(float) 0.036952,
(float) 0.095533,
(float) 0.068936,
(float)-0.000000,
(float)-0.050404,
(float)-0.050835,
(float)-0.014169,
(float) 0.023083,
(float) 0.033543,
(float) 0.016774,
(float)-0.007466,
(float)-0.019340,
(float)-0.013755,
(float) 0.000000,
(float) 0.009400,
(float) 0.009029,
(float) 0.002381,
(float)-0.003658,
(float)-0.005027,
(float)-0.002405,
(float) 0.001050,
(float) 0.002780,
(float) 0.002145,
(float) 0.000000
};


/*****************************************************************************
/*
/*	  Function to find out the adaptive codebook vector [A.3.7.1]
/*    Input: u[240] excitation signal 
/*           T1, frac1: Optimal integer and fractional part of pitch delay for the 1st subframe
/*			 T2, frac2: Optimal integer and fractional part of pitch delay for the 2nd subframe
/*    Output: v[80] adaptive codebook vector
/*
/****************************************************************************/
void adaptive_codebook_vector(float u[200], int T1, int frac1, float v[40])
{
	int n, i;
	int k, t;
	int zero_index;

	if (frac1 == -1)
	{
		t = 2;
		k = T1 - 1;
	}
	else
	{
		t = frac1;
		k = T1;
	}
	zero_index = 160 - k;
	for (n=0; n<40; n++)
	{
		for (i=0, v[n]=0; i<=9; i++)
			v[n] += u[n+zero_index+i]*b30[t+3*i];
		for (i=0; i<=9; i++)
			v[n] += u[n+zero_index+1+i]*b30[3-t+3*i];
	}
}
